import { Options } from 'lru-cache';
import { RedisOptions } from 'ioredis';
import { PoolConfig } from 'pg';

interface TileCacheOptions {
    /**
     * @description Flag which indicate if the cache should be enabled. Default is true.
     */
    enabled?: boolean;
    /**
     * @deprecated replaced by {enabled}
     */
    enable?: boolean;
    /**
     * @description The type of the cache. Default is lru-cache
     */
    type?: 'lru-cache' | 'redis';
    /**
     * @description LRU cache options
     */
    lruOptions?: Options;
    /**
     * @description Redis connect options
     */
    redisOptions?: RedisOptions & {
        /**
         * @description The time to live in seconds. Default is 86400 (1 day)
         */
        ttl?: number;
    };
}

/**
 * The specification of the tile request
 */
interface TileRequest {
    /**
     * @description The zoom level ranging from 0 - 20
     */
    z: number;
    /**
     * @description The tile x offset on Mercator Projection
     */
    x: number;
    /**
     * @description The tile y offset on Mercator Projection
     */
    y: number;
}

/**
 * @description The required input values for the tile renderer
 */
interface TileInput<T> extends TileRequest {
    /**
     * @description The name of the table, default is "public.points"
     */
    table?: string;
    /**
     * @description The geometry column name, default is "wkb_geometry". This column should be of type Geometry in PostGIS
     */
    geometry?: string;
    /**
     * @description The MVT source layer on which the points are rendered, default is points
     */
    sourceLayer?: string;
    /**
     * @description The cluster radius in pixels. Default is 20
     */
    radius?: number;
    /**
     * @description The tile extent is the grid dimension value as specified by ST_AsMVT. The default is 4096.
     * @see https://postgis.net/docs/ST_AsMVT.html
     */
    extent?: number;
    /**
     * @description The buffer around the tile extent in the number of grid cells as specified by ST_AsMVT. The default is 256.
     * @see https://postgis.net/docs/ST_AsMVT.html
     */
    bufferSize?: number;
    /**
     * @description The query parameters used to filter
     */
    queryParams?: T | {};
    /**
     * @description Unique ID of the request, default is an empty string
     */
    id?: string;
}

/**
 * @description This function creates the MVT tiles from the appropriate TileInput
 */
declare type TileRenderer<T> = (args: TileInput<T>) => Promise<ArrayBuffer>;

/**
 * @description Configuration options for the tile server
 */
interface TileServerConfig<T> {
    /**
     * @description The highest zoom level at which data is clustered.
     * Any tile requests at zoom levels higher than this will return individual points only.
     */
    maxZoomLevel?: number;
    /**
     * @description The tile resolution in pixels, default is 512, but try 256 if you
     * are unsure what your mapping front-end library uses
     * @deprecated This is ignored and will be removed in future releases
     */
    resolution?: number;
    /**
     * @description LRU tile cache options, each tile request is stored in this cache.
     * clusterbuster tries to provide sane defaults
     */
    cacheOptions?: TileCacheOptions;
    /**
     * @description Configuration options for the postgres connection pool
     * clusterbuster tries to provide sane defaults
     */
    pgPoolOptions?: PoolConfig;
    /**
     * @description Optional callback to map the filters to where conditions in PostGreSQL
     */
    filtersToWhere?: (queryParams: T | {}) => string[];
    /**
     * @description Attributes to select from the table
     */
    attributes: string[];
    /**
     * @description Show debug logging, default false
     */
    debug?: boolean;
}

declare function TileServer<T>({ maxZoomLevel, resolution, cacheOptions, pgPoolOptions, filtersToWhere, attributes, debug, }: TileServerConfig<T>): Promise<TileRenderer<T>>;

export { TileCacheOptions, TileInput, TileRenderer, TileRequest, TileServer, TileServerConfig };
